package webdb53.tinycal.action;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

import org.seasar.cubby.action.ActionClass;
import org.seasar.cubby.action.ActionResult;
import org.seasar.cubby.action.Form;
import org.seasar.cubby.action.Json;
import org.seasar.cubby.action.RequestParameter;
import org.seasar.cubby.action.Validation;
import org.seasar.cubby.validator.ConversionValidationRule;
import org.seasar.cubby.validator.ValidationRules;
import org.seasar.cubby.validator.validators.RequiredValidator;
import org.seasar.framework.beans.util.Beans;
import org.seasar.framework.container.annotation.tiger.Binding;

import webdb53.tinycal.entity.Schedule;
import webdb53.tinycal.service.ScheduleService;

/**
 * カレンダー用のAPI
 * @author agata
 */
@ActionClass
public class ScheduleAction {
	@Binding
	public ScheduleService scheduleService;
	
	// list
	@RequestParameter
	public Date from;
	@RequestParameter
	public Date to;
	
	// update
	@RequestParameter
	public Integer id;
	@RequestParameter
	public Timestamp datetime;
	@RequestParameter
	public String title;
	@RequestParameter
	public String memo;

	public ValidationRules listValidationRules = new JsonSupportedValidationRules("schedule.") {
		protected void initialize(String resourceKeyPrefix) {
			add("from", new RequiredValidator());
			add("to", new RequiredValidator());
		};
	};
	
	public ValidationRules updateValidationRules = new JsonSupportedValidationRules("schedule.") {
		protected void initialize(String resourceKeyPrefix) {
			add("datetime", new RequiredValidator());
			add("title", new RequiredValidator());
		};
	};
	
	public ValidationRules removeValidationRules = new JsonSupportedValidationRules("schedule.") {
		protected void initialize(String resourceKeyPrefix) {
			add("id", new RequiredValidator());
		};
	};
	
	/**
	 * from/toを指定してスケジュール一覧取得
	 * @return スケジュール一覧 Json<Schedule>
	 */
	@Form
	@Validation(rules="listValidationRules")
	public ActionResult list() {
		List<Schedule> result = scheduleService.findByDate(from, to);
		return new Json(result);
	}

	/**
	 * スケジュールの追加、更新
	 * スケジュールIDがnullなら追加、それ以外は更新
	 * @return Json<null>
	 */
	@Form
	@Validation(rules="updateValidationRules")
	public ActionResult update() {
		Schedule schedule = Beans.createAndCopy(Schedule.class, this).dateConverter("yyyy/MM/dd HH:mm:ss", "datetime").execute();
		scheduleService.update(schedule);
		return new Json("OK");
	}

	/**
	 * スケジュールIDを指定してスケジュールの削除
	 * @return Json<null>
	 */
	@Form
	@Validation(rules="removeValidationRules")
	public ActionResult remove() {
		scheduleService.remove(id);
		return new Json("OK");
	}
}