/*
 * Copyright 2004-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package webdb53.tinycal.action;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.seasar.cubby.action.ActionErrors;
import org.seasar.cubby.action.ActionResult;
import org.seasar.cubby.action.Json;
import org.seasar.cubby.plugins.s2.unit.CubbyTestCase;

import webdb53.tinycal.RunDdlServletRequestListener;
import webdb53.tinycal.entity.Schedule;

public class ScheduleActionTest extends CubbyTestCase {

	public ScheduleAction action;

	public ActionErrors actionErrors;

	protected void setUp() throws Exception {
		include("app.dicon");
		RunDdlServletRequestListener runDsl = new RunDdlServletRequestListener();
		runDsl.requestInitialized(null);
	}

	public void testScheduleListValidationError() throws Exception {
		ActionResult result = processAction("/schedule/list");
		assertTrue(result instanceof Json);
		assertEquals(2, actionErrors.getAll().size());
		assertEquals("fromは必須です。", actionErrors.getFields().get("from").get(0));
		assertEquals("toは必須です。", actionErrors.getFields().get("to").get(0));
	}

	public void testScheduleUpdateValidationError() throws Exception {
		ActionResult result = processAction("/schedule/update");
		assertTrue(result instanceof Json);
		assertEquals(2, actionErrors.getAll().size());
		assertEquals("予定日時は必須です。", actionErrors.getFields().get("datetime").get(0));
		assertEquals("タイトルは必須です。", actionErrors.getFields().get("title").get(0));

		actionErrors.clear();
		getRequest().addParameter("datetime", "dummy");
		result = processAction("/schedule/update");
		assertTrue(result instanceof Json);
		assertEquals(1, actionErrors.getFields().size());
		assertEquals("タイトルは必須です。", actionErrors.getFields().get("title").get(0));
	}

	@SuppressWarnings("unchecked")
	public void testScheduleListSuccess() throws Exception {
		getRequest().addParameter("from", "2009/10/01");
		getRequest().addParameter("to", "2009/10/15");
		ActionResult result = processAction("/schedule/list");
		assertTrue(result instanceof Json);
		assertEquals(0, actionErrors.getAll().size());
		List<Schedule> schedules = (List<Schedule>) ((Json)result).getBean();
		assertEquals(0, schedules.size());
	}

	@SuppressWarnings({ "unchecked", "deprecation" })
	public void testScheduleAddAndListAndRemove() throws Exception {
		getRequest().addParameter("datetime", "2009/10/10 12:30:00");
		getRequest().addParameter("title", "WEB+DB PRESS原稿レビュー");
		getRequest().addParameter("memo", "参加者：縣、桜井、稲尾");
		
		ActionResult result = processAction("/schedule/update");
		assertTrue(result instanceof Json);
		assertEquals("OK", ((Json)result).getBean());
		
		getRequest().getParameterMap().clear();
		getRequest().addParameter("from", "2009/10/01");
		getRequest().addParameter("to", "2009/10/15");
		result = processAction("/schedule/list");
		assertTrue(result instanceof Json);
		List<Schedule> schedules = (List<Schedule>) ((Json)result).getBean();
		assertEquals(1, schedules.size());
		assertNotNull(schedules.get(0).getId());
		SimpleDateFormat format = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		assertEquals(format.format(new Date(2009 - 1900, 9, 10, 12, 30)), format.format(schedules.get(0).getDatetime()));
		assertEquals("WEB+DB PRESS原稿レビュー", schedules.get(0).getTitle());
		assertEquals("参加者：縣、桜井、稲尾", schedules.get(0).getMemo());
		
		getRequest().getParameterMap().clear();
		getRequest().addParameter("id", schedules.get(0).getId().toString());
		result = processAction("/schedule/remove");
		assertTrue(result instanceof Json);
		assertEquals("OK", ((Json)result).getBean());

		getRequest().getParameterMap().clear();
		getRequest().addParameter("from", "2009/10/01");
		getRequest().addParameter("to", "2009/10/15");
		result = processAction("/schedule/list");
		assertTrue(result instanceof Json);
		schedules = (List<Schedule>) ((Json)result).getBean();
		assertEquals(0, schedules.size());
	}
}
